@@ -549,6 +549,34 @@ gp_fopen (const char * fname, const char * mode) {
 
 }
 
+int gp_can_share_fdesc(void)
+{
+    return 0;
+}
+
+FILE *gp_open_scratch_file_rm(const gs_memory_t *mem,
+                              const char        *prefix,
+                                    char         fname[gp_file_name_sizeof],
+                              const char        *mode)
+{
+    return NULL;
+}
+
+FILE *gp_fdup(FILE *f, const char *mode)
+{
+    return NULL;
+}
+
+int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
+{
+    return -1;
+}
+
+int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
+{
+    return -1;
+}
+
 FILE *
 popen (const char * fname, const char * mode ) {
         return gp_fopen (fname,  mode);
@@ -662,7 +690,7 @@ const char *gp_file_name_current(void)
 {   return ":";
 }
 
-bool gp_file_name_is_partent_allowed(void)
+bool gp_file_name_is_parent_allowed(void)
 {   return true;
 }
 
