@@ -24,6 +24,7 @@
 #include "gxband.h"
 #include "gxbcache.h"
 #include "gxclio.h"
+#include "gxcolor2.h"		/* for pattern1_instance */
 #include "gxdevbuf.h"
 #include "gxistate.h"
 #include "gxrplane.h"
@@ -59,11 +60,6 @@
  * questions is negative, we flush the buffer.
  */
 
-#ifndef gs_pattern1_instance_t_DEFINED
-#  define gs_pattern1_instance_t_DEFINED
-typedef struct gs_pattern1_instance_s gs_pattern1_instance_t;
-#endif
-
 /* ---------------- Public structures ---------------- */
 
 /*
@@ -75,9 +71,20 @@ typedef struct gs_pattern1_instance_s gs_pattern1_instance_t;
  */
 typedef struct gx_saved_page_s {
     char dname[32];		/* device name for checking */
-    gx_device_color_info color_info;
-    gx_band_page_info_t info;
-    int num_copies;
+    gx_device_color_info color_info;	/* also for checking */
+    /* Elements from gx_band_page_info that we need */
+    char cfname[gp_file_name_sizeof];	/* command file name */
+    char bfname[gp_file_name_sizeof];	/* block file name */
+    const clist_io_procs_t *io_procs;
+    uint tile_cache_size;	/* size of tile cache */
+    int64_t bfile_end_pos;		/* ftell at end of bfile */
+    gx_band_params_t band_params;  /* parameters used when writing band list */
+                                /* (actual values, no 0s) */
+    gs_memory_t *mem;		/* allocator for paramlist */
+    /* device params are expected to include info to set the icc_struct and the devn_params */
+    /* as well as color_info (if it is able to change as the 'bit' devices can).            */
+    int paramlist_len;
+    byte *paramlist;		/* serialized device param list */
 } gx_saved_page;
 
 /*
@@ -119,6 +126,7 @@ typedef struct {
     /*   0 means unused */
     /* reading: offset from cdev->chunk.data */
 } tile_hash;
+
 typedef struct {
     gx_cached_bits_common;
     /* To save space, instead of storing rep_width and rep_height, */
@@ -175,9 +183,7 @@ typedef struct gx_clist_state_s gx_clist_state;
         uint data_size;			/* size of buffer */\
         gx_band_params_t band_params;	/* band buffering parameters */\
         bool do_not_open_or_close_bandfiles;	/* if true, do not open/close bandfiles */\
-        bool page_uses_transparency;	/* if true then page uses PDF 1.4 transparency */\
-        int is_planar;                  /* if non zero then we have a planar device
-                                         * with is_planar bits per component. */\
+        int is_printer;                 /* if true, then clist is based on a prn device */\
                 /* Following are used for both writing and reading. */\
         gx_bits_cache_chunk chunk;	/* the only chunk of bits */\
         gx_bits_cache bits;\
@@ -195,13 +201,9 @@ typedef struct gx_clist_state_s gx_clist_state;
         clist_icctable_t *icc_table;    /* Table that keeps track of ICC profiles.\
                                            It relates the hashcode to the cfile\
                                            file location. */\
-        gsicc_link_cache_t *icc_cache_cl  /* Link cache */
-
-/*
- * Chech whether a clist is used for storing a pattern command stream.
- * Useful for both reader and writer.
- */
-#define IS_CLIST_FOR_PATTERN(cdev) (cdev->procs.open_device == pattern_clist_open_device)
+        gsicc_link_cache_t *icc_cache_cl; /* Link cache */\
+        int icc_cache_list_len;         /* Length of list of caches, one per rendering thread */\
+        gsicc_link_cache_t **icc_cache_list  /* Link cache list */
 
 /* Define a structure to hold where the ICC profiles are stored in the clist
    Profiles are added into psuedo bands of the clist, these are bands that exist beyond
@@ -215,17 +217,14 @@ typedef struct gx_clist_state_s gx_clist_state;
 typedef struct clist_icc_serial_entry_s clist_icc_serial_entry_t;
 
 struct clist_icc_serial_entry_s {
-
     int64_t hashcode;              /* A hash code for the icc profile */
     int64_t file_position;        /* File position in cfile of the profile with header */
     int size;
-
 };
 
 typedef struct clist_icctable_entry_s clist_icctable_entry_t;
 
 struct clist_icctable_entry_s {
-
     clist_icc_serial_entry_t serial_data;
     clist_icctable_entry_t *next;  /* The next entry in the table */
     cmm_profile_t *icc_profile;    /* The profile.  In non-gc memory. This is
@@ -251,10 +250,13 @@ struct clist_icctable_s {
     clist_icctable_entry_t *final;
 };
 
+void
+clist_icc_table_finalize(const gs_memory_t *memory, void * vptr);
+
 #define private_st_clist_icctable()\
-  gs_private_st_ptrs2(st_clist_icctable,\
+  gs_private_st_ptrs2_final(st_clist_icctable,\
                 clist_icctable_t, "clist_icctable",\
-                clist_icctable_enum_ptrs, clist_icctable_reloc_ptrs, head, final)
+                clist_icctable_enum_ptrs, clist_icctable_reloc_ptrs, clist_icc_table_finalize, head, final)
 
 typedef struct gx_device_clist_common_s {
     gx_device_clist_common_members;
@@ -290,6 +292,7 @@ typedef struct clist_icc_color_s {
     int64_t icc_hash;           /* hash code for icc profile */
     byte icc_num_components;   /* needed to avoid having to read icc data early */
     bool is_lab;               /* also needed early */
+    gsicc_profile_t default_match;     /* used by gsicc_get_link early for usefastcolor mode */
     gsicc_colorbuffer_t data_cs;
 } clist_icc_color_t;
 
@@ -328,6 +331,7 @@ struct gx_device_clist_writer_s {
                                 /* the page level BM, shape or opacity alpha needs tranaparency */
     int pdf14_trans_group_level;/* 0 when at page level group -- push increments, pop decrements */
     int pdf14_smask_level;	/* 0 when at SMask None -- push increments, pop decrements */
+    bool page_pdf14_needed;	/* save page level pdf14_needed state */
 
     float dash_pattern[cmd_max_dash];	/* current dash pattern */
     const gx_clip_path *clip_path;	/* current clip path, */
@@ -495,6 +499,14 @@ int clist_render_rectangle(gx_device_clist *cdev,
 /* This function updates the clist writer states with the bbox provided. */
 void clist_update_trans_bbox(gx_device_clist_writer *dev, gs_int_rect *bbox);
 
+/* Make a clist device for accumulating. Used for pattern-clist as well as */
+/* for pdf14 pages that are too large to be done in page mode.             */
+gx_device_clist *
+clist_make_accum_device(gx_device *target, const char *dname, void *base, int space,
+                        gx_device_buf_procs_t *buf_procs, gx_band_params_t *band_params,
+                        bool use_memory_clist, bool uses_transparency,
+                        gs_pattern1_instance_t *pinst);
+
 /* Retrieve total size for cfile and bfile. */
 int clist_data_size(const gx_device_clist *cdev, int select);
 /* Get command list data. */
@@ -526,7 +538,7 @@ int clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode,
                        cmm_profile_t *icc_profile);
 
 /* Free the table and its entries */
-int clist_icc_freetable(clist_icctable_t *icc_table, gs_memory_t *memory);
+int clist_free_icc_table(clist_icctable_t *icc_table, gs_memory_t *memory);
 
 /* Generic read function used with ICC and could be used with others.
    A different of this and clist_get_data is that here we reset the
