@@ -665,8 +665,8 @@ ztempfile(i_ctx_t *i_ctx_p)
     const char *pstr;
     char fmode[4];
     int code = parse_file_access_string(op, fmode);
-    char prefix[gp_file_name_sizeof];
-    char fname[gp_file_name_sizeof];
+    char *prefix = NULL;
+    char *fname= NULL;
     uint fnlen;
     FILE *sfile;
     stream *s;
@@ -674,6 +674,13 @@ ztempfile(i_ctx_t *i_ctx_p)
 
     if (code < 0)
         return code;
+    prefix = (char *)gs_alloc_bytes(imemory, gp_file_name_sizeof, "ztempfile(prefix)");
+    fname = (char *)gs_alloc_bytes(imemory, gp_file_name_sizeof, "ztempfile(fname)");
+    if (!prefix || !fname) {
+        code = gs_note_error(gs_error_VMerror);
+        goto done;
+    }
+
     strcat(fmode, gp_fmode_binary_suffix);
     if (r_has_type(op - 1, t_null))
         pstr = gp_scratch_file_name_prefix;
@@ -682,8 +689,10 @@ ztempfile(i_ctx_t *i_ctx_p)
 
         check_read_type(op[-1], t_string);
         psize = r_size(op - 1);
-        if (psize >= gp_file_name_sizeof)
-            return_error(e_rangecheck);
+        if (psize >= gp_file_name_sizeof) {
+            code = gs_note_error(e_rangecheck);
+            goto done;
+        }
         memcpy(prefix, op[-1].value.const_bytes, psize);
         prefix[psize] = 0;
         pstr = prefix;
@@ -692,29 +701,37 @@ ztempfile(i_ctx_t *i_ctx_p)
     if (gp_file_name_is_absolute(pstr, strlen(pstr))) {
         if (check_file_permissions(i_ctx_p, pstr, strlen(pstr),
                                    "PermitFileWriting") < 0) {
-            return_error(e_invalidfileaccess);
+            code = gs_note_error(e_invalidfileaccess);
+            goto done;
         }
     } else if (!prefix_is_simple(pstr)) {
-        return_error(e_invalidfileaccess);
+        code = gs_note_error(e_invalidfileaccess);
+        goto done;
     }
 
     s = file_alloc_stream(imemory, "ztempfile(stream)");
-    if (s == 0)
-        return_error(e_VMerror);
+    if (s == 0) {
+        code = gs_note_error(e_VMerror);
+        goto done;
+    }
     buf = gs_alloc_bytes(imemory, file_default_buffer_size,
                          "ztempfile(buffer)");
-    if (buf == 0)
-        return_error(e_VMerror);
+    if (buf == 0) {
+        code = gs_note_error(e_VMerror);
+        goto done;
+    }
     sfile = gp_open_scratch_file(imemory, pstr, fname, fmode);
     if (sfile == 0) {
         gs_free_object(imemory, buf, "ztempfile(buffer)");
-        return_error(e_invalidfileaccess);
+        code = gs_note_error(e_invalidfileaccess);
+        goto done;
     }
     fnlen = strlen(fname);
     sbody = ialloc_string(fnlen, ".tempfile(fname)");
     if (sbody == 0) {
         gs_free_object(imemory, buf, "ztempfile(buffer)");
-        return_error(e_VMerror);
+        code = gs_note_error(e_VMerror);
+        goto done;
     }
     memcpy(sbody, fname, fnlen);
     file_init_stream(s, sfile, fmode, buf, file_default_buffer_size);
@@ -724,10 +741,17 @@ ztempfile(i_ctx_t *i_ctx_p)
         sclose(s);
         iodev_dflt->procs.delete_file(iodev_dflt, fname);
         ifree_string(sbody, fnlen, ".tempfile(fname)");
-        return_error(e_VMerror);
+        code = gs_note_error(e_VMerror);
+        goto done;
     }
     make_string(op - 1, a_readonly | icurrent_space, fnlen, sbody);
     make_stream_file(op, s, fmode);
+
+done:
+    if (prefix)
+        gs_free_object(imemory, prefix, "ztempfile(prefix)");
+    if (fname)
+        gs_free_object(imemory, fname, "ztempfile(fname)");
     return code;
 }
 
@@ -834,8 +858,11 @@ zopen_file(i_ctx_t *i_ctx_p, const gs_parsed_file_name_t *pfn,
 {
     gx_io_device *const iodev = pfn->iodev;
 
-    if (pfn->fname == NULL)     /* just a device */
+    if (pfn->fname == NULL) {     /* just a device */
+        iodev->state = i_ctx_p;
         return iodev->procs.open_device(iodev, file_access, ps, mem);
+        iodev->state = NULL;
+    }
     else {                      /* file */
         iodev_proc_open_file((*open_file)) = iodev->procs.open_file;
 
@@ -903,18 +930,21 @@ lib_file_open_search_with_no_combine(gs_file_path_ptr  lib_path, const gs_memory
 {
     stream *s;
     uint blen1 = blen;
+    struct stat fstat;
+
     if (gp_file_name_reduce(fname, flen, buffer, &blen1) != gp_combine_success)
       goto skip;
-    if (iodev_os_open_file(iodev, (const char *)buffer, blen1,
+
+    if ((*iodev->procs.file_status)(iodev,  buffer, &fstat) >= 0) {
+        if (starting_arg_file || check_file_permissions_aux(i_ctx_p, buffer, blen1) >= 0) {
+            if (iodev_os_open_file(iodev, (const char *)buffer, blen1,
                            (const char *)fmode, &s, (gs_memory_t *)mem) == 0) {
-      if (starting_arg_file ||
-          check_file_permissions_aux(i_ctx_p, buffer, blen1) >= 0) {
-        *pclen = blen1;
-        make_stream_file(pfile, s, "r");
-        return 0;
-      }
-      sclose(s);
-      return_error(e_invalidfileaccess);
+                *pclen = blen1;
+                make_stream_file(pfile, s, "r");
+                return 0;
+            }
+        }
+        return_error(e_invalidfileaccess);
     }
  skip:;
     return 1;
@@ -1000,11 +1030,6 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
     gs_main_instance *minst = get_minst_from_memory(mem);
     int code;
 
-    /* Once we've opened the arg file, prevent searching current dir from now on */
-    /* If this causes problems with "strange" scripts and utlities, we have to rethink */
-    if (i_ctx_p)
-        i_ctx_p->starting_arg_file = false;
-
     /* when starting arg files (@ files) iodev_default is not yet set */
     if (iodev == 0)
         iodev = (gx_io_device *)gx_io_device_table[0];
