@@ -31,6 +31,7 @@
 #include "gsparam.h"	    /* these are needed to check device parameters */
 #include "zfunc.h"
 #include "zcolor.h"
+#include "gxdevsop.h"
 
 /*
  * FunctionType 4 functions are not defined in the PostScript language.  We
@@ -195,7 +196,7 @@ put_op(byte **p, byte op) {
  * known to be a procedure.
  */
 static int
-check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int *psize)
+check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int *psize, bool AllowRepeat)
 {
     int code;
     uint i, j;
@@ -373,28 +374,11 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
             if ((code = array_get(imemory, pref, ++i, &elt2)) < 0)
                 return code;
             *psize += 3;
-            code = check_psc_function(i_ctx_p, &elt, depth + 1, ops, psize);
+            code = check_psc_function(i_ctx_p, &elt, depth + 1, ops, psize, AllowRepeat);
             if (code < 0)
                 return code;
             /* Check for { proc } repeat | {proc} if | {proc1} {proc2} ifelse */
             if (resolves_to_oper(i_ctx_p, &elt2, zrepeat)) {
-                gs_c_param_list list;
-                int AllowRepeat = 1;
-
-                /* Check if the device allows the use of repeat in functions */
-                /* We can't handle 'repeat' with pdfwrite since it emits FunctionType 4 */
-                gs_c_param_list_write(&list, i_ctx_p->pgs->device->memory);
-                code = gs_getdeviceparams(i_ctx_p->pgs->device, (gs_param_list *)&list);
-                if (code < 0)
-                    return code;
-                gs_c_param_list_read(&list);
-                code = param_read_bool((gs_param_list *)&list,
-                    "AllowPSRepeatFunctions",
-                    &AllowRepeat);
-                if (code < 0)
-                    return code;
-                gs_c_param_list_release(&list);
-
                 if (!AllowRepeat)
                     return_error(e_rangecheck);
                 if (ops) {
@@ -421,7 +405,7 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
                     *p = PtCr_else;
                 }
                 *psize += 3;
-                code = check_psc_function(i_ctx_p, &elt2, depth + 1, ops, psize);
+                code = check_psc_function(i_ctx_p, &elt2, depth + 1, ops, psize, AllowRepeat);
                 if (code < 0)
                     return code;
                 if (ops)
@@ -450,6 +434,7 @@ gs_build_function_4(i_ctx_t *i_ctx_p, const ref *op, const gs_function_params_t
     int code;
     byte *ops;
     int size;
+    int AllowRepeat = 1; /* Default to permitting Repeat, devices which can't handle it implement the spec_op */
 
     *(gs_function_params_t *)&params = *mnDR;
     params.ops.data = 0;	/* in case of failure */
@@ -463,7 +448,33 @@ gs_build_function_4(i_ctx_t *i_ctx_p, const ref *op, const gs_function_params_t
         goto fail;
     }
     size = 0;
-    code = check_psc_function(i_ctx_p, proc, 0, NULL, &size);
+
+    /* Check if the device allows the use of repeat in functions */
+    /* We can't handle 'repeat' with pdfwrite since it emits FunctionType 4 */
+    {
+        char data[] = {"AllowPSRepeatFunctions"};
+        dev_param_req_t request;
+        gs_c_param_list list;
+
+        gs_c_param_list_write(&list, i_ctx_p->pgs->device->memory);
+        /* Stuff the data into a structure for passing to the spec_op */
+        request.Param = data;
+        request.list = &list;
+        code = dev_proc(i_ctx_p->pgs->device, dev_spec_op)(i_ctx_p->pgs->device, gxdso_get_dev_param, &request, sizeof(dev_param_req_t));
+        if (code < 0 && code != gs_error_undefined) {
+            gs_c_param_list_release(&list);
+            return code;
+        }
+        gs_c_param_list_read(&list);
+        code = param_read_bool((gs_param_list *)&list,
+            "AllowPSRepeatFunctions",
+            &AllowRepeat);
+        gs_c_param_list_release(&list);
+        if (code < 0)
+            return code;
+    }
+
+    code = check_psc_function(i_ctx_p, proc, 0, NULL, &size, AllowRepeat);
     if (code < 0)
         goto fail;
     ops = gs_alloc_string(mem, size + 1, "gs_build_function_4(ops)");
@@ -472,7 +483,7 @@ gs_build_function_4(i_ctx_t *i_ctx_p, const ref *op, const gs_function_params_t
         goto fail;
     }
     size = 0;
-    check_psc_function(i_ctx_p, proc, 0, ops, &size); /* can't fail */
+    check_psc_function(i_ctx_p, proc, 0, ops, &size, AllowRepeat); /* can't fail */
     ops[size] = PtCr_return;
     params.ops.data = ops;
     params.ops.size = size + 1;
@@ -493,6 +504,7 @@ int make_type4_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t *
     float *ptr;
     ref alternatespace, *palternatespace = &alternatespace;
     PS_colour_space_t *space, *altspace;
+    int AllowRepeat = 1; /* Default to permitting Repeat, devices which can't handle it implement the spec_op */
 
     code = get_space_object(i_ctx_p, arr, &space);
     if (code < 0)
@@ -542,14 +554,40 @@ int make_type4_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t *
     params.ops.data = 0;	/* in case of failure, see gs_function_PtCr_free_params */
     params.ops.size = 0;	/* ditto */
     size = 0;
-    code = check_psc_function(i_ctx_p, (const ref *)pproc, 0, NULL, &size);
+
+    /* Check if the device allows the use of repeat in functions */
+    /* We can't handle 'repeat' with pdfwrite since it emits FunctionType 4 */
+    {
+        char data[] = {"AllowPSRepeatFunctions"};
+        dev_param_req_t request;
+        gs_c_param_list list;
+
+        gs_c_param_list_write(&list, i_ctx_p->pgs->device->memory);
+        /* Stuff the data into a structure for passing to the spec_op */
+        request.Param = data;
+        request.list = &list;
+        code = dev_proc(i_ctx_p->pgs->device, dev_spec_op)(i_ctx_p->pgs->device, gxdso_get_dev_param, &request, sizeof(dev_param_req_t));
+        if (code < 0 && code != gs_error_undefined) {
+            gs_c_param_list_release(&list);
+            return code;
+        }
+        gs_c_param_list_read(&list);
+        code = param_read_bool((gs_param_list *)&list,
+            "AllowPSRepeatFunctions",
+            &AllowRepeat);
+        gs_c_param_list_release(&list);
+        if (code < 0)
+            return code;
+    }
+
+    code = check_psc_function(i_ctx_p, (const ref *)pproc, 0, NULL, &size, AllowRepeat);
     if (code < 0) {
         gs_function_PtCr_free_params(&params, imemory);
         return code;
     }
     ops = gs_alloc_string(imemory, size + 1, "make_type4_function(ops)");
     size = 0;
-    check_psc_function(i_ctx_p, (const ref *)pproc, 0, ops, &size); /* can't fail */
+    check_psc_function(i_ctx_p, (const ref *)pproc, 0, ops, &size, AllowRepeat); /* can't fail */
     ops[size] = PtCr_return;
     params.ops.data = ops;
     params.ops.size = size + 1;
